// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"time"

	"gorm.io/gorm"
)

// 学位
type Dws24 struct {
	ID         int64  `gorm:"primary_key" json:"-"`
	DegreeId   string `json:"degree_id"`
	Name       string `json:"name"`
	BirthDate  string `json:"birth_date"`
	Gender     string `json:"gender"`
	Major      string `json:"major"`
	DegreeName string `json:"degree_name"`
	AwardName  string `json:"award_name"`
	AwardDate  string `json:"award_date"`
	CreatedAt  int64  `json:"-"`
	UpdatedAt  int64  `json:"-"`
}

func (Dws24) TableName() string {
	return "t_adm_dws24"
}

// Insert 插入一条记录
func (p *Dws24) Insert(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	return err
}

func (p *Dws24) InsertByMap(db *gorm.DB, data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Create(data).Error
	return err
}

func (p *Dws24) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	return err
}

func (p *Dws24) Save(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.UpdatedAt = timeNow
	//db.Model().Update()
	err := db.Save(p).Error
	return err
}

func (p *Dws24) Update(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.CreatedAt = 0
	p.UpdatedAt = timeNow
	err := db.Model(p).Updates(p).Error
	return err
}

func (p *Dws24) UpdateByMap(db *gorm.DB, data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Updates(data).Error
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *Dws24) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	/*if filed == nil {
		return errors.ParamsError
	}*/
	timeNow := time.Now().Unix()
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	return err
}

func (p *Dws24) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if p.ID == 0 {
		return gorm.ErrRecordNotFound
	}

	return err
}

func (p *Dws24) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error

	return err
}
